/*
 * Szegedi Tudományegyetem
 * Informatikai Tanszékcsoport
 * Szoftverfejlesztés Tanszék
 */

#include "pnm.h"

#include <dlfcn.h>
#include <malloc.h>
#include <stdio.h>
#include <string.h>

/*
 * A main függvény.
 */
int main(int argc, char *argv[]) {
    int argi;
    void *lib_handle;
    char *error_msg;
    rgbpic_t *kep = NULL, *tmp = NULL;

    for (argi = 1; argi < argc; ++argi) {
        rgbpic_plugin_start_fnc start;
        rgbpic_plugin_work_fnc work;
        rgbpic_plugin_stop_fnc stop;

        char *next_par = argv[argi];
        char *lib_name = NULL;
        char *lib_pars = NULL;
        size_t name_size;

        for (lib_pars = next_par; *lib_pars; ++lib_pars) {
            if (*lib_pars == '=') {
                *(lib_pars++) = '\0';
                break;
            }
        }
        name_size = strlen(next_par) + 7;
        lib_name = malloc(name_size * sizeof(char));
        sprintf(lib_name, "lib%s.so", next_par);

        lib_handle = dlopen(lib_name, RTLD_LAZY);
        if (!lib_handle) {
            fprintf(stderr, "Nem sikerült megnyitni a %s fájlt!\n%s\n", lib_name, dlerror());
            free(lib_name);
            continue;
        }
        free(lib_name);

        start = dlsym(lib_handle, RGBPIC_PLUGIN_START_FNC_NAME);
        if ((error_msg = dlerror()) != NULL) {
            fprintf(stderr, "%s\n", error_msg);
            dlclose(lib_handle);
            rgbpic_delete(kep);
            return 1;
        }
        if (start(lib_pars)) {
            fprintf(stderr, "Nem sikerült elindítani a %s plugint!\n", next_par);
            dlclose(lib_handle);
            break;
        };

        work = dlsym(lib_handle, RGBPIC_PLUGIN_WORK_FNC_NAME);
        if ((error_msg = dlerror()) != NULL) {
            fprintf(stderr, "%s\n", error_msg);
            dlclose(lib_handle);
            rgbpic_delete(kep);
            return 1;
        }
        tmp = work(kep);
        if (tmp == NULL) {
            fprintf(stderr, "A %s plugin futtatása nem sikerült!\n", next_par);
            dlclose(lib_handle);
            break;
        }
        kep = tmp;

        stop = dlsym(lib_handle, RGBPIC_PLUGIN_STOP_FNC_NAME);
        if ((error_msg = dlerror()) != NULL) {
            fprintf(stderr, "%s\n", error_msg);
            dlclose(lib_handle);
            rgbpic_delete(kep);
            return 1;
        }
        if (stop(lib_pars)) {
            fprintf(stderr, "Nem sikerült lezárni a %s plugint!\n", next_par);
            dlclose(lib_handle);
            break;
        };

        dlclose(lib_handle);
    }

    rgbpic_delete(kep);
    return 0;
}
