#include "pnm.h"
#include <stdlib.h>
#include <string.h>
#include <time.h>

#define TRANSFERLUM(PIX,TF,TW,FF,FW,MAX) while (PIX->FF > TW && PIX->TF < MAX - FW) { PIX->FF -= TW; PIX->TF += FW; }

static void fuzz_rg(rgbpic_pixel_t *p, rgbpic_monochrome_pixel_t m) {
    TRANSFERLUM(p,red,30,green,59,m)
}

static void fuzz_rb(rgbpic_pixel_t *p, rgbpic_monochrome_pixel_t m) {
    TRANSFERLUM(p,red,30,blue,11,m)
}

static void fuzz_gr(rgbpic_pixel_t *p, rgbpic_monochrome_pixel_t m) {
    TRANSFERLUM(p,green,59,red,30,m)
}

static void fuzz_gb(rgbpic_pixel_t *p, rgbpic_monochrome_pixel_t m) {
    TRANSFERLUM(p,green,59,blue,11,m)
}

static void fuzz_br(rgbpic_pixel_t *p, rgbpic_monochrome_pixel_t m) {
    TRANSFERLUM(p,blue,11,red,30,m)
}

static void fuzz_bg(rgbpic_pixel_t *p, rgbpic_monochrome_pixel_t m) {
    TRANSFERLUM(p,blue,11,green,59,m)
}

typedef void (*fuzz_t)(rgbpic_pixel_t*, rgbpic_monochrome_pixel_t);
static const fuzz_t fuzz[] = {
    fuzz_rg,
    fuzz_rb,
    fuzz_gr,
    fuzz_gb,
    fuzz_br,
    fuzz_bg,
    NULL
};
static int iterations = 0;

int RGBPIC_PLUGIN_START_FNC(const char *params) {
    srand(time(0));
    if (params == NULL || !strcmp("", params)) {
        iterations = 1024;
    } else {
        iterations = atoi(params);
    }
    return 0;
}

rgbpic_t* RGBPIC_PLUGIN_WORK_FNC(rgbpic_t *pic) {
    int fuzz_size;
    for (fuzz_size = 0; fuzz[fuzz_size]; ++fuzz_size);
    int n = pic->width * pic->height;
    while (iterations--) {
        rgbpic_picture_size_t x, y, dx, dy;
        x  = random() % pic->width;
        y  = random() % pic->height;
        dx = (pic->width / 32) + random() % (pic->width / 8);
        dy = (pic->height / 32) + random() % (pic->height / 8);
        fuzz_t f = fuzz[random() % fuzz_size];
        int idx = 0;
        for (rgbpic_picture_size_t i = y; i < y + dy && idx < n; ++i) {
            idx = rgbpic_coord(pic, x, i);
            for (rgbpic_picture_size_t j = x; j < x + dx && idx < n ; ++j, ++idx) {
                if (idx < n) {
                    f(pic->pixelvalues + idx, pic->maxpixvalue);
                }
            }
        }
    }
    return pic;
}

int RGBPIC_PLUGIN_STOP_FNC() {
    return 0;
}
